<?php

namespace Drupal\taxonomy_vocabulary_tree\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\taxonomy\TermInterface;
use Drupal\taxonomy_vocabulary_tree\Service\VocabularyTreeRepository;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a block.
 *
 * @Block(
 *  id = "taxonomy_vocabulary_tree_block",
 *  admin_label = @Translation("Taxonomy vocabulary tree block"),
 * )
 */
class SliderNamesTreeBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;


  /**
   * Constructs a new SystemBreadcrumbBlock object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Drupal\taxonomy_vocabulary_tree\Service\VocabularyTreeRepository $taxonomy_vocabulary_tree_manager
   *   The module handler service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ModuleHandlerInterface $module_handler,
    VocabularyTreeRepository $taxonomy_vocabulary_tree_manager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->moduleHandler = $module_handler;
    $this->taxonomyVocabularyTreeManager = $taxonomy_vocabulary_tree_manager;

  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    /*  @noinspection  PhpParamsInspection */
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('module_handler'),
      $container->get('taxonomy_vocabulary_tree.repository')
    );
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Exception
   */
  public function build() {
    $config = $this->getConfiguration();
    if (empty($config['vocabulary_name'])) {
      return $this->warningsMessage();
    }


    $tree = $this->taxonomyVocabularyTreeManager->loadHierarchyTree(
      $config['vocabulary_name']
    );



    $build = $this->buildTerms($tree);
    $build['#attached']['library'][] = 'taxonomy_vocabulary_tree/menu-tree';
    $build['#attributes']['class'][] = 'taxonomy-menu-tree';

    return $build;
  }

  /**
   * Show warning message.
   *
   * @return array
   *   The status messages.
   */
  protected function warningsMessage() {
    $this->messenger()->addWarning(
      $this->t('Please select vocabulary_name in configuration block.')
    );

    return [
      '#type' => 'status_messages',
    ];
  }

  /**
   * Build terms.
   *
   * @param array $terms
   *   The term.
   *
   * @return array
   *   The renderable array.
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  protected function buildTerms(array $terms) {
    $menu_classes = $this->getConfiguration()['menu_classes'] ?? '';

    $build = [
      '#type' => 'container',
      '#theme_wrappers' => [],
      '#prefix' => '<ul class="' . $menu_classes . ' menu-tree-main-level menu-tree-nested">',
      '#suffix' => '</ul>',
    ];

    foreach ($terms as $term) {
      $build[] = $this->buildTerm($term);
    }

    return $build;
  }

  /**
   * Build term.
   *
   * @param array $term
   *   The term.
   *
   * @return array
   *   Renderable array.
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  protected function buildTerm($term) {
    $config = $this->getConfiguration();
    $build = [
      '#type' => 'inline_template',
      '#template' => '<li class="{{ item_classes }} without-nesting"> {{ html_prefix }}  {{ term_name }} </li>',
      '#context' => [
        'item_classes' => $config['item_classes'] ?? '',
        'html_prefix' => [
          '#markup' => $config['without_nesting_html_prefix'] ?? '',
        ],
        'term_name' => [
          '#markup' => $this->buildTermName($term['entity']),
        ],
      ],
    ];

    if (empty($term['children'])) {
      return $build;
    }
    else {
      $children = [];
      foreach ($term['children'] as $child) {
        $children[] = $this->buildTerm($child);
      }

      $build['#template'] = '<li class="{{ item_classes }}"> {{ html_prefix }}  {{ term_name }} <ul class="menu-tree-nested">';
      $build['#context']['html_prefix']['#markup'] = $config['html_prefix'] ?? '';

      return [
        '#type' => 'container',
        '#theme_wrappers' => [],
        'item' => $build,
        'children' => $children,
        '#suffix' => '</ul></li>',
      ];
    }
  }

  /**
   * Build term name.
   *
   * @param \Drupal\taxonomy\TermInterface $term
   *   The term.
   *
   * @return string
   *   The term name link.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  protected function buildTermName(TermInterface $term) {
    $config = $this->getConfiguration();
    $use_url_from_term = $config['use_url_from_term'] ?? FALSE;

    if ($use_url_from_term) {
      $term_url = $term->toUrl()->toString();
    }
    else {
      $url_template = $this->getConfiguration()['term_url'] ?? '';
      $term_url = str_replace('[tid]', $term->id(), $url_template);
    }


    return '<a href="' . $term_url . '">' . $term->getName() . '</a>';
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $vocabulary_names = taxonomy_vocabulary_get_names();

    $config = $this->getConfiguration();
    $form['vocabulary_name'] = [
      '#type' => 'select',
      '#title' => $this->t('Vocabulary name'),
      '#options' => $vocabulary_names,
      '#default_value' => $config['vocabulary_name'] ?? NULL,
      '#required' => TRUE,
    ];

    $form['use_url_from_term'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use url from term entity.'),
    ];

    $form['term_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Term url'),
      '#default_value' => $config['term_url'] ?? '/taxonomy/term/[tid]',
      '#maxlength' => 255,
      '#description' => $this->t('You can use [tid] for term id in url.'),
      '#required' => TRUE,
      '#states' => [
        'visible' => [
          ':input[name="settings[use_url_from_term]"]' => [
            'checked' => FALSE,
          ],
        ],
      ],
    ];

    $form['html_prefix'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Html prefix'),
      '#default_value' => $config['html_prefix'] ?? '<i class="menu-tree-caret"></i>',
      '#maxlength' => 255,
      '#description' => $this->t('The html prefix before menu item.'),
    ];

    $form['without_nesting_html_prefix'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Without nesting html prefix'),
      '#default_value' => $config['without_nesting_html_prefix'] ?? '<i class="menu-tree-caret-without-nesting"></i>',
      '#maxlength' => 255,
      '#description' => $this->t('The html prefix before menu item without nesting.'),
    ];

    $form['menu_classes'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Menu classes'),
      '#default_value' => $config['menu_classes'] ?? 'list-group categories',
      '#maxlength' => 255,
      '#description' => $this->t('The css classes of menu.'),
    ];

    $form['item_classes'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Menu item classes'),
      '#default_value' => $config['item_classes'] ?? 'list-group-item',
      '#maxlength' => 255,
      '#description' => $this->t('The css classes of menu item.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    foreach ($form_state->getValues() as $name => $value) {
      $this->setConfigurationValue($name, $value);
    }
  }

}
