<?php

namespace Drupal\taxonomy_vocabulary_tree\Service;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Implements BlazyManagerInterface, SlickManagerInterface.
 */
class VocabularyTreeRepository implements ContainerInjectionInterface {

  /**
   * Term storage.
   *
   * @var \Drupal\taxonomy\TermStorageInterface
   */
  protected $termStorage;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Creates a new VocabularyTreeService.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    LanguageManagerInterface $language_manager,
    ModuleHandlerInterface $module_handler
  ) {
    $this->termStorage = $entity_type_manager->getStorage(
      'taxonomy_term'
    );
    $this->languageManager = $language_manager;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public static function create(ContainerInterface $container) {
    /*  @noinspection  PhpParamsInspection */
    return new static(
      $container->get('entity_type.manager'),
      $container->get('language_manager'),
      $container->get('module_handler')
    );
  }

  /**
   * Load hierarchy tree in a given vocabulary ID.
   *
   * @param string $vid
   *   Vocabulary ID to retrieve terms for.
   * @param int $parent
   *   The term ID under which to generate the tree. If 0, generate the tree
   *   for the entire vocabulary.
   * @param int $max_depth
   *   The number of levels of the tree to return. Leave NULL to return all
   *   levels.
   * @param bool $langcode
   *   The langcode to translate the terms.
   *
   * @return object[]|\Drupal\taxonomy\TermInterface[]
   *   An array of term objects that are the children of the vocabulary $vid.
   */
  public function loadHierarchyTree(
    string $vid,
    int $parent = 0,
    int $max_depth = NULL,
    string $langcode = ''
  ) {
    $terms = $this->termStorage->loadTree(
      $vid,
      $parent,
      $max_depth,
      TRUE
    );
    $translated_terms = $this->getTermTranslations($terms, $langcode);
    $tree = $this->structuringTree($translated_terms);

    $this->moduleHandler->alter(
      'taxonomy_vocabulary_tree',
      $tree,
      $vid
    );

    return $tree;
  }

  /**
   * Finds all translated terms in a given vocabulary ID.
   *
   * @param string $vid
   *   Vocabulary ID to retrieve terms for.
   * @param int $parent
   *   The term ID under which to generate the tree. If 0, generate the tree
   *   for the entire vocabulary.
   * @param int $max_depth
   *   The number of levels of the tree to return. Leave NULL to return all
   *   levels.
   * @param bool $langcode
   *   The langcode to translate the terms.
   *
   * @return object[]|\Drupal\taxonomy\TermInterface[]
   *   An array of term objects that are the children of the vocabulary $vid.
   */
  public function loadTree(
    string $vid,
    int $parent = 0,
    int $max_depth = NULL,
    string $langcode = ''
  ) {
    $terms = $this->termStorage->loadTree(
      $vid,
      $parent,
      $max_depth,
      TRUE
    );
    $translated_terms = $this->getTermTranslations($terms, $langcode);

    return $translated_terms;
  }

  /**
   * Finds all children of a term ID.
   *
   * The children will be translated to current language.
   *
   * @param int $tid
   *   Term ID to retrieve children for.
   * @param string $vid
   *   An optional vocabulary ID to restrict the child search.
   *
   * @return \Drupal\taxonomy\TermInterface[]
   *   An array of translated term objects that are the children of the term $tid.
   */
  public function loadChildren(int $parent_id, string $vocabulary_name) {
    $terms = $this->termStorage->loadChildren($parent_id, $vocabulary_name);
    $translated_terms = $this->getTermTranslations($terms);

    return $translated_terms;
  }

  /**
   * Translating terms.
   *
   * @param array $terms
   * @param string $langcode
   *
   * @return \Drupal\taxonomy\Entity\Term[]
   *   The translated terms.
   */
  public function getTermTranslations(array $terms, string $langcode = '') {
    if (empty($langcode)) {
      $langcode = $this->languageManager->getCurrentLanguage()->getId();
    }
    $translated_terms = [];

    /** @var \Drupal\taxonomy\Entity\Term $term */
    foreach ($terms as $term) {
      if ($term->hasTranslation($langcode)) {
        $translated_terms[$term->id()] = $term->getTranslation($langcode);
      }
      else {
        $translated_terms[$term->id()] = $term;
      }
    }

    return $translated_terms;
  }

  /**
   * Structuring vocabulary tree.
   *
   * @param array $terms
   *   The terms.
   *
   * @return array
   *   The structured tree.
   */
  protected function structuringTree(array $terms) {
    $tree = [];
    $depth = 0;
    $links = [0 => &$tree];
    $link = $links[0];
    foreach ($terms as $term) {
      $parent_id = $term->get('parent')->getString();
      if ($term->depth > $depth) {
        $link = &$link['children'][$parent_id];
        $links[$parent_id] = &$link;
      }
      $depth = $term->depth;

      unset($link);
      $link = &$links[$parent_id];
      $link['children'][$term->id()]['entity'] = $term;
    }

    return $tree['children'];
  }

}
